using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;
using System.Threading;
using System.Diagnostics;
using System.IO;

namespace WebCapture_WebRole
{
    public class WebRole : RoleEntryPoint
    {
        public override void Run()
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var container = account.CreateCloudBlobClient().GetContainerReference("output");
            container.CreateIfNotExist();
            container.SetPermissions(new BlobContainerPermissions { PublicAccess = BlobContainerPublicAccessType.Blob });

            var queue = account.CreateCloudQueueClient().GetQueueReference("incoming");
            queue.CreateIfNotExist();

            var outputPath = Path.Combine(RoleEnvironment.GetLocalResource("LocalOutput").RootPath, "output.png");

            while (true)
            {
                var msg = queue.GetMessage();
                if (msg != null)
                {
                    if (msg.DequeueCount < 3)
                    {
                        var split = msg.AsString.Split(new char[] { '/' }, 2);
                        var guid = split[0];
                        var url = split[1];
                        var proc = new Process()
                        {
                            StartInfo = new ProcessStartInfo(Environment.GetEnvironmentVariable("RoleRoot") + @"\\approot\CutyCapt.exe",
                                    string.Format(@"--url=""{0}"" --out=""{1}""",
                                        url,
                                        outputPath))
                                {
                                    UseShellExecute = false
                                }
                        };
                        proc.Start();
                        proc.WaitForExit();
                        if (File.Exists(outputPath))
                        {
                            var blob = container.GetBlobReference(guid);
                            blob.Properties.ContentType = "image/png";
                            blob.UploadFile(outputPath);
                            File.Delete(outputPath);
                        }
                    }
                    queue.DeleteMessage(msg);
                }
                else
                {
                    Thread.Sleep(TimeSpan.FromSeconds(1));
                }
            }
        }

        public override bool OnStart()
        {
            DiagnosticMonitor.Start("DiagnosticsConnectionString");

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            RoleEnvironment.Changing += RoleEnvironmentChanging;

            return base.OnStart();
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            // If a configuration setting is changing
            if (e.Changes.Any(change => change is RoleEnvironmentConfigurationSettingChange))
            {
                // Set e.Cancel to true to restart this role instance
                e.Cancel = true;
            }
        }
    }
}
